#
# depgraph.rb
#
# Data structures to represent the dependency graph
# of resolvables.
#

#
# DepEntry, connects a resolvable with a specific dependency
#

class DepEntry
  attr_reader :dependency, :resolvable

  def initialize( dep, res )
    @dependency = dep
    @resolvable = res
  end

  def to_s
    return "#{@resolvable.to_s}[#{@dependency.to_s}]"
  end

end

class DepGraph
  attr_reader :provides, :requires, :prerequires, :conflicts, :obsoletes

  def initialize
    @provides = Hash.new			# "name" => resolvable
    @requires = Hash.new
    @prerequires = Hash.new
    @conflicts = Hash.new
    @obsoletes = Hash.new
  end

  #
  # return list of DepEntry(s) providing kind:tag
  #

  def providers kind, tag
#STDERR.puts "DepGraph.providers( #{kind}:#{tag} )" if $debug
	all = @provides[tag] || []
	result = []
#STDERR.puts "DepGraph.providers( #{kind}:#{tag} -> #{all.size} hits for #{tag} )" # if $debug
	all.each do |entry|
#STDERR.puts "? #{entry.to_s}"
	    result << entry if entry.resolvable.kind == kind
	end
#STDERR.puts "found #{result.size} providers" if $debug
	return result
  end

  #
  # add a resolvable to the graph
  #
  def add resolvable
    fill( resolvable, resolvable.provides, @provides )
    fill( resolvable, resolvable.requires, @requires )
    fill( resolvable, resolvable.prerequires, @prerequires )
  end

  #
  # remove a resolvable from the graph
  #
  def remove resolvable
  end

  private

  #
  # fill resolvables dependencies into container
  #

  def fill( resolvable, dependencies, container )
    return if dependencies.nil?
    dependencies.each { |dep|
      name = dep.name
      entry = DepEntry.new( dep, resolvable )
      container[name] = [] unless container[name]
      container[name] << entry
    }

  end

end
