#
# input/database
# reads package data from an activerecord database
#

require "zypptools/lib/database"
require "zypptools/lib/db"
require "zypptools/lib/repo"
require "zypptools/lib/package"
#require "zypptools/lib/patch"
require "zypptools/lib/pattern"
require "zypptools/lib/product"

class Database

  # read complete repo from database

  def read_repo
    repo = Repo.new

    # match reskind to object attribute
    {:package => :packages, :pattern => :patterns, :patch => :patches, :product => :products}.each do |kind,var|
      reskind = Db::Reskind.get kind.id2name
      repo.send( var.id2name + "=", read_bykind( reskind ) )
    end
    return repo
  end

  private

  # read resolvables matching reskind

  def read_bykind reskind
    result = Hash.new
    resolvable = nil
    (Db::Resolvable.find_all_by_kind_id reskind.id).each do |res|
      Resolvable.from_db res
      result[res.id] = resolvable
    end
    return result
  end

end

class Dependency

  def Dependency.from_db resolvable, id
    deps = Array.new
    (Db::Dependency.find_all_by_resolvable_id id, :include => [:relation, :deptype]).each do |dep|
#puts "---#{deptype.name}:#{resolvable.name}---"
#pp dep
#puts "---------"
      dependency = Dependency.new dep.name, dep.refers.name.to_sym, dep.pre
      if dep.relation
        dependency.op = dep.relation.name
        dependency.version = dep.version
        dependency.release = dep.release if dep.release
      end
      deps = resolvable.send( dep.deptype.name )
      deps << dependency
      resolvable.send( dep.deptype.name+"=", deps )
    end
  end

end

class Resolvable

  def Resolvable.from_db res
    case res.kind.name
        when "package"
          resolvable = Package.new( res.name, res.epoch, res.version, res.release, res.architecture.name )
        when "pattern"
          resolvable = Pattern.new( res.name, res.epoch, res.version, res.release, res.architecture.name )
        when "patch"
          resolvable = Patch.new( res.name, res.epoch, res.version, res.release, res.architecture.name )
        when "product"
          resolvable = Product.new( res.name, res.epoch, res.version, res.release, res.architecture.name )
        else
          raise "Loaded unknown reskind #{reskind.name}"
      end
    deps = Dependency.from_db resolvable, res["id"]

    return resolvable
  end

end
