require 'rubygems'
require 'rpm'
require 'zypptools/input/rpmfile'
require 'pathname'

#
# input plugin for a directory with
# rpms
#
class RPMDir
  def initialize( logger, path, recursive = false )
    @recursive = recursive
    @logger = logger
    @path = path
  end

  # we are a consumer for RPMFile
  # and we proxy to the real consumer
  def consume( package )
    #puts "consuming #{package}"
    @logger.info("Consuming #{package}")
    @consumer.consume(package)
  end

  def consume_dir(path)
    Dir.entries(path).each do | file |
      @logger.info file
      if File.extname(file) == ".rpm"
        rpmfile = RPMFile.new((Pathname.new(path) + file).to_s)
        # we are the consumer
        rpmfile.start(self)
      end
      if File.directory?("#{path}/#{file}") and @recursive
        next if file == "." or file == ".."
        @logger.info "Found subdirectory #{path}/#{file}"
        consume_dir("#{path}/#{file}")
      end
    end
  end

  def start( consumer )
    @consumer = consumer
    #puts "no"
    @consumer.start
    #puts "yes"
    consume_dir(@path)
  end

  def finish
    @consumer.finish
  end
end

