require 'rubygems'
require 'rpm'
require "zypptools/lib/arch"
require "zypptools/lib/package"
require "zypptools/lib/hasher"

#
# Input plugin for a single rpm file
#

#
# Adapter class that simulates a repotools
# dependency interface from a RPM dependency
#
class RPMDependencyAdapter < Dependency
  # attr_accessor :name, :refers, :op, :epoch, :version, :release, :pre
  def initialize( rpm_dep )
    @rpm_dep = rpm_dep
  end

  def method_missing( sym, *args)
    return @rpm_dep.send( sym )
  end

  def name
    return @rpm_dep.name
  end

  def refers
    return :package
  end

  def pre
    return @rpm_dep.pre?
  end

  def version
    @rpm_dep.version.v
  end

  def release
    @rpm_dep.version.r
  end

  def epoch
    @rpm_dep.version.e
  end

  def op
    return "==" if eq?
    return "<=" if le?
    return ">=" if ge?
    return "<" if lt?
    return ">" if gt?
    return nil
  end
end

#
# Adapter class that simulates a repotools
# Package interface from a RPM Package
#
class RPMPackageAdapter

  attr_accessor :filename

  def initialize( rpm_package, filename )
    @rpm_package = rpm_package
    @filename = filename
  end

  def kind
    return :package
  end

  def version
    @rpm_package.version.v
  end

  def release
    @rpm_package.version.r
  end

  def epoch
    @rpm_package.version.e
  end

  def sha1sum
    h = Hasher.new("sha1", @filename)
    h.hashsum
  end

  def pre
    pre?
  end

  def method_missing( sym, *args)
    case sym.to_s
      when 'group'
        return @rpm_package[RPM::TAG_GROUP]
      when 'license'
        return @rpm_package[RPM::TAG_LICENSE]
      when 'description'
        return @rpm_package[RPM::TAG_DESCRIPTION]
      when 'summary'
        return @rpm_package[RPM::TAG_SUMMARY]
      when 'url'
        return @rpm_package[RPM::TAG_URL]
      when 'buildtime'
        return @rpm_package[RPM::TAG_BUILDTIME]
      when 'buildhost'
        return @rpm_package[RPM::TAG_BUILDHOST]
      when 'size'
        return @rpm_package[RPM::TAG_SIZE]
      when 'archivesize'
        return @rpm_package[RPM::TAG_ARCHIVESIZE]
      when 'vendor'
        return @rpm_package[RPM::TAG_VENDOR]
      when 'distribution'
        return @rpm_package[RPM::TAG_DISTRIBUTION]
      when 'sourcerpm'
        return @rpm_package[RPM::TAG_SOURCERPM]
      when 'sigsize'
        return @rpm_package[RPM::TAG_SIGSIZE]
      when 'provides', 'requires','conflicts', 'obsoletes'
        a = Array.new
        @rpm_package.send(sym).each do | dep |
          a << RPMDependencyAdapter.new(dep)
        end
        return a
    end

    if @rpm_package.respond_to?(sym)
      begin
        return @rpm_package.send( sym )
      rescue Exception => e
        puts e
        puts "Method #{sym} : args : #{args.to_s} : #{RPM::TAG_URL}"
      end
      raise "oops!"
    end
  end

  def architecture
    self.arch
  end
end

class RPMFile
  def initialize( filename )
    @filename = filename
  end

  def start( consumer )
    rpm_package = RPM::Package.new(@filename)
    adapter = RPMPackageAdapter.new(rpm_package, @filename)
    consumer.consume(adapter)
  end

  def stop

  end
end

