#
# TagParser for 'tagfile' style files (packages, packages.*, *.pat)
#

class TagParser

	attr_reader :filename, :lnum

	#
	# TagParser constructor
	# takes filename
	#

	def initialize( filename )
		@filename = filename
		begin	# rescue block
			File.open( filename ) do |f|
				@f = f
				parse_internal
				self.send( :finish )
			end
		rescue
			STDERR.puts "Parsing of #{@filename} failed"
			raise;
		end
	end

	# -----------------------------------------------------

	protected

	#
	# skip everything until next occurence of tag (or eof)
	#

	def skip_until tag
		@skip = tag
#		STDERR.puts "Skipping until #{tag}"
	end

	# -----------------------------------------------------

	private

	def get_line
		l = ""
		loop do
			return nil if @f.eof?
			l = @f.gets.chomp
			@lnum = @lnum + 1
#puts "get_line(#{@lnum}:#{l})"
			next if l.size == 0
			next if l[0,1] == "#"
			break
		end
		return l
	end

	def parse_internal
		l = ""
		@lnum = 0
		loop do
			l = get_line
			break unless l
			c = l[0,1]
#puts "parse_internal([#{c}]#{l})"
			case c
				when "+": parse_tag( l, true )
				when "=": parse_tag( l, false )
				else next
			end
		end
	end

	#
	# parse line containing a tag
	#  multi == true if its a multi-line tag (+Xxx ... -Xxx)
	#

	def parse_tag( line, multi )
		tag,value = line.split( ":" )
		tag = tag[1..-1]
		if multi then					# multi value tag, collect all values
			endtag = "-#{tag}:"
			value = []
			loop do
				l = get_line
				return nil unless l
				break if l == endtag
				value << l
			end
		else
			value.strip! unless value.nil?
		end
		tag, @lang = tag.split( "." )
		@skip = nil if @skip && @skip == tag		# reset @skip on next occurence of tag
#		STDERR.puts "Skipping #{tag}" if @skip
		self.send( tag, value ) unless @skip
	end

end

