
#<package pkgid="3be01d71919acaab4b4f5b07b1636c866006c469" name="gtklp-debuginfo" arch="i586"><version epoch="0" ver="1.2.2" rel="43"/><file>/usr/lib/debug/usr/bin/gtklp.debug</file><file>/usr/lib/debug/usr/bin/gtklpq.debug

require 'zlib'
require "rexml/document"

require 'sqlite3'

include REXML  # so that we don't have to prefix everything with REXML::...

# TODO move this out of here

class DBBase
  def initialize(dbname, schema)
    @dbname = dbname
    exist = File.exists?(@dbname)
    @db = SQLite3::Database.new(@dbname)
    if not exist
      sql = File.new(schema).read
      @db.execute_batch( sql )
    end
  end

  def start
    @db.execute('begin;')
  end

  def finish
    @db.execute('commit;')
  end

end

#@insert_dir_stmt = @db.prepare( "insert into dirs ( name, version, release, epoch, arch, kind, summary, description) values ( ?, ?, ?, ?, ? ,? , ?, ? )" )

class DBFileListConsumer4 < DBBase

  def initialize(dbname)
    super(dbname, "/space/sources/zypp/trunk/libzypp/zypp2/cache/schema/schema_filelist4.sql")
    @dircache = Hash.new
    @insert_dirnames_stmt = @db.prepare( "insert into dir_names ( name ) values ( ? );" )
    @insert_filenames_stmt = @db.prepare( "insert into file_names ( name ) values ( ? );" )
    @insert_file_stmt = @db.prepare( "insert into files ( dir_id, file_id, package_id ) values ( ?, ?, ? )" )
  end

  def consume_entry( pkgid, entry )
    dir = File.dirname(entry)
    return if dir == "."
    if ( @dircache[dir].nil? )
      dir_id = insert_dir(dir);
      @dircache.clear
      @dircache[dir] = dir_id
    else
      dir_id = @dircache[dir]
    end
    filename = File.basename(entry)
    file_id = insert_file(filename)
    begin
      @insert_file_stmt.execute( dir_id, file_id, pkgid )
    rescue Exception => e
      puts "Error at file insert: #{pkgid} #{dir_id} #{dir} #{file_id} #{filename }from '#{entry}'"
    end
  end

  def insert_dir( dir )
    v = @db.get_first_value("select id from dir_names where name=?", dir)
    if v.nil?
       begin
        @insert_dirnames_stmt.execute dir
       rescue Exception => e
        puts "Error at dir insert: #{dir}"
       end
       return @db.last_insert_row_id
    else
      return v
    end
  end

  def insert_file( file )
    v = @db.get_first_value("select id from file_names where name=?", file)
    if v.nil?
       begin
        @insert_filenames_stmt.execute file
       rescue Exception => e
        puts "Error at filename insert: #{file}"
       end
       return @db.last_insert_row_id
    else
      return v
    end
  end
end

class DBFileListConsumer2 < DBBase

  def initialize(dbname)
    super(dbname, "/space/sources/zypp/trunk/libzypp/zypp2/cache/schema/schema_filelist2.sql")

    @insert_dir_stmt = @db.prepare( "insert into dirs ( name ) values ( ? );" )
    @insert_file_stmt = @db.prepare( "insert into files ( dir_id, package_id, name  ) values ( ?, ?, ? )" )
  end

  
  def consume_entry( pkgid, entry )
    dir = File.dirname(entry)
    return if dir == "."
    dir_id = insert_dir(dir);
    filename = File.basename(entry)
    begin
      @insert_file_stmt.execute( dir_id, pkgid, filename )
    rescue Exception => e
      puts "Error at package insert: #{pkgid} #{dir_id} #{dir} #{filename} from '#{entry}'"
    end
  end

  def insert_dir( dir )
    v = @db.get_first_value("select id from dirs where name=?", dir)
    if v.nil?
       begin
        @insert_dir_stmt.execute dir
       rescue
        puts "Error at dir insert: #{dir}"
       end
       return @db.last_insert_row_id
    else
      return v
    end
  end
end


class FileListListener

    def initialize(consumer)
       @in_file = false
       @consumer = consumer
       @num = 0
       @buffer = String.new
    end

    def tag_start(name, attrs)   
      #puts "+#{name}"
      case name
      when 'package'
        @files = Array.new
        @in_file = false
        @num = @num + 1
        @id = attrs['id']
        @name = attrs['name']
        @arch = attrs['arch']
      when 'version'
        @version = attrs['ver']
        @release = attrs['rel']
        @epoch = attrs['epoch']
      when 'file'
        @in_file = true
      end
    end

    def tag_end(name)   
      #puts "-#{name}"
      case name
      when 'file'
        @files.push(@buffer) if @in_file
        @consumer.consume_entry(@num, @buffer)
        @buffer = String.new
        @in_file = false
      when 'package'
        #puts "#{@name} has #{@files.size} files"
        
      end
    end

    def text(t)
      #puts t
      #puts @in_file
      #puts @files.class.to_s
      @buffer = @buffer + t
    end

    def method_missing( symbol, *params )
    end
end



class FileEntryProducer
  def initialize( filename, consumer )
    @consumer = consumer
    @filename = filename
  end

  def start
    listener = FileListListener.new(@consumer)
    Zlib::GzipReader.open(@filename) do |gz|
      REXML::Document.parse_stream(gz, listener)
    end
  end
end

db = DBFileListConsumer4.new(ARGV[1])
db.start
ep = FileEntryProducer.new(ARGV[0], db)
ep.start
db.finish

# -> health
# -> food
#doc.elements.each("*/section/item") { |element| puts element.attributes["upc"] }
