#
# lib/db
# resolvable data in an activerecord database
#

require "rubygems"
require "pp"
require_gem "activerecord"

module DbGet
  @@cache = Hash.new
  def get key
    obj = @@cache[key]
    return obj if obj
    if key.class == String
      #STDERR.print "find_by_name(#{key})"
      obj = find_by_name( key )
    elsif key.class == Integer
      #STDERR.print "find_by_id(#{key})"
      obj = find_by_id( key )
    else
      raise "Can't find key '#{key}' [#{key.class}] in #{self.name}"
    end
    raise "Couldn't find #{self.name} '#{key}' [#{key.class}]" unless obj
    @@cache[key] = obj
    return obj
  end
end


class Db
  class Architecture < ActiveRecord::Base
    extend DbGet
  end
    
  class Relation < ActiveRecord::Base
    extend DbGet
  end
  
  class Reskind < ActiveRecord::Base
    extend DbGet
  end
    
  class Deptype < ActiveRecord::Base
    extend DbGet
    has_many :dependencies
  end
  
  class Dependency < ActiveRecord::Base
    belongs_to :deptype
    belongs_to :relation
    belongs_to :resolvable
    belongs_to :refers, :class_name => "Reskind", :foreign_key => "refers_id"
  end
  
  class Resolvable < ActiveRecord::Base
    has_many :dependencies
    belongs_to :kind, :class_name => "Reskind", :foreign_key => "kind_id"
    belongs_to :architecture
  end
  
  class Transaction < ActiveRecord::Base
    belongs_to :resolvable, :foreign_key => 'id'
  end
  
  class Catalog < ActiveRecord::Base
  end
end # class db
