#
# Representation of a dependency
#

class Dependency
  attr_accessor :name, :refers, :op, :epoch, :version, :release, :pre

  #
  # parse - parse array of dependencies
  #
  def Dependency.parse( ary, refers, pre = false )
    deps = []
    ary.each do | depstr |
      next if depstr[0,7] == "rpmlib("
      deps << Dependency.new( depstr, refers, pre )
    end
    return deps
  end

  # returns a symbol with the operator
  # code
  # <  lt
  # <= le
  def flag
    return :ge if op == '>='
    return :gt if op == '>'
    return :lt if op == '<'
    return :le if op == '<='
    return :eq if op == '=='
    return :ne if op == '!='
  end

  def init_with_hash( properties )
    @pre = properties[:pre]
    @refers = properties[:refers]
    @name = properties[:name]
    @op = properties[:op]
    @release = properties[:release]
    @version = properties[:version]
    @release = properties[:release]
  end
  # parse "name op e:v-r"
  #  evra = epoch, version, release
  def initialize( str, refers = nil, pre = false )
    #puts "init dependency with hash: #{str.class.to_s}"
    if str.class.to_s == "Hash"
     init_with_hash str
    else
      @pre = pre
      @refers = refers
      splitted = str.split " "
      if ((splitted.size < 2)  ||          # just a name
          !(splitted[1].match( /[<=>]/ )))
        @name = str
        @op = nil
        return
      end
      @name = splitted[0]
  
      raise( "bad dependency [#{str}]" ) unless splitted.size == 3  # ensure <op> and <evra>
  
      @op = splitted[1]
      @op = "==" if @op == "="
      evr = splitted[2].split "-"          # check if <version>-<release>
  
      if evr.size < 2              # just <version>
        @version = splitted[2]
        @release = nil
        return
      end
      @release = evr[1]
  
      ev = evr[0].split ":"            # check if <epoch>:<version>
      if ev.size < 2
        @version = evr[0]
      else
        @epoch = ev[0]
        @version = ev[1]
      end
    end
  end

  def to_s
    s = "<dep name='#{name}'"
    s += " refers='#{refers.id2name}'" if refers
    if op
      s += " op='#{op}' version='#{version}'"
      s += " release='#{release}'" if release
    end
    return s + "/>"
  end

end

