#
# Representation of a Resolvable
#

require 'zypptools/lib/dependency'

class Resolvable
  attr_accessor :kind, :name, :epoch, :version, :release, :architecture
  attr_accessor :chk_type, :chk_sum
  attr_accessor :summary, :description
  attr_accessor :requires, :prerequires, :provides, :conflicts, :obsoletes
  attr_accessor :suggests, :enhances, :supplements, :recommends, :freshens
  attr_accessor :buildtime

  alias arch architecture

  def initialize( kind, name, epoch = nil, version = nil, release = nil, architecture = nil )
                @kind = kind
                @name = name
                @epoch = epoch
                @version = version
                @release = release
                @architecture = architecture

    # ensure that the resolvable provides itself
  end

  def add_self_provide
    selfprovides = Dependency.new( @name, @kind )
    if @version
      selfprovides.op = "=="
      selfprovides.epoch = @epoch
      selfprovides.version = @version
      selfprovides.release = @release
    end
    @provides = Array.new if @provides.nil?
    @provides.push selfprovides
  end

  def deps depname, ary, refers = nil, pre = false
    raise "Resolvable.deps called without refers" if refers.nil?
    current = self.send( depname.id2name )
    unless current
      self.send( depname.id2name + "=", [] )
      current = []
    end
    self.send( depname.id2name + "=", current.concat( Dependency.parse( ary, refers, pre ) ) )
  end

  def edition
    s = String.new
    s += "#{@version}" if @version
    s += "-#{@release}" if @release
    return s
  end

  def to_s
    s = "#{kind.id2name}[#{@name}"
    s += "-#{@version}" if @version
    s += "-#{@release}" if @release
    s += ".#{@architecture}" if @architecture
    return s + "]"
  end

end

