#
# output/database
# writes package data to an activerecord database
#

require "zypptools/lib/db"
require "zypptools/lib/database"

class Database

  def create


	begin
	 ActiveRecord::Base.connection.execute( "drop table architectures" )
	 ActiveRecord::Base.connection.execute( "drop table relations" )
	 ActiveRecord::Base.connection.execute( "drop table deptypes" )
	 ActiveRecord::Base.connection.execute( "drop table reskinds" )
	 ActiveRecord::Base.connection.execute( "drop table dependencies" )
	 ActiveRecord::Base.connection.execute( "drop table resolvables" )
	 ActiveRecord::Base.connection.execute( "drop table transactions" )
	 ActiveRecord::Base.connection.execute( "drop table catalogs" )
	rescue
	end

	ActiveRecord::Base.connection.execute( "create table architectures (
		id integer not null default 0 primary key autoincrement,
		name string)" )
	ActiveRecord::Base.connection.execute( "create table relations (
		id integer not null default 0 primary key autoincrement,
		name string,
		op integer)" )
	ActiveRecord::Base.connection.execute( "create table deptypes (
		id integer not null default 0 primary key autoincrement,
		name string)" )
	ActiveRecord::Base.connection.execute( "create table reskinds (
		id integer not null default 0 primary key autoincrement,
		name string)" )

	ActiveRecord::Base.connection.execute( "create table dependencies (
		id integer not null default 0 primary key autoincrement,
		deptype_id integer not null,
		resolvable_id integer not null,
		name string not null,
		refers_id integer not null,
		relation_id integer,
		epoch integer,
		version string,
		release string,
		pre integer default 0)" )

	ActiveRecord::Base.connection.execute( "create table resolvables (
		id integer not null default 0 primary key autoincrement,
		kind_id integer not null,
		name string not null,
		epoch integer,
		version string,
		release string,
		catalog string,
		architecture_id integer)" )

  ActiveRecord::Base.connection.execute( "create table transactions (
		action integer not null,
		id integer not null,
		details string )" )

  ActiveRecord::Base.connection.execute( "CREATE TABLE catalogs (  id VARCHAR PRIMARY KEY,  name VARCHAR,  alias VARCHAR,  description VARCHAR,  priority INTEGER,  subscribed INTEGER,  checksum VARCHAR,  timestamp INTEGER)" )

	# Init architectures

	["noarch", "i386", "i486", "i586", "i686", "x86_64", "ppc", "ppc64",
	 "ia64", "s390", "s390x"].each do |arch|

	  Db::Architecture.create( :name => arch )

	end


	# Init relations
	# op is a bitmask, bit0: ==, bit1: <, bit2: >

	Db::Relation.create( [
	 { :name => "==", :op => 1 },
	 { :name => "<",  :op => 2 },
	 { :name => ">",  :op => 4 },
	 { :name => "<=", :op => 3 },
	 { :name => ">=", :op => 5 },
	 { :name => "!=", :op => 6 }
	] )


	# Init reskinds

	["package", "patch", "pattern", "product"].each do |kind|
	  Db::Reskind.create( :name => kind )
	end

	# Init deptypes

	["provides", "requires", "conflicts", "obsoletes",
	 "recommends", "suggests",
	 "essentialfor", "supplements", "enhances",
	 "freshens"].each do |type|
	  Db::Deptype.create( :name => type )
	end

#	ActiveRecord::Base.connection.execute( "commit" )

  end # create()


  def save_repo( repo, catalog = nil )
    ActiveRecord::Base.connection.execute( "begin" )
    repo.packages.each_value { |p|
      p.to_db( self, catalog )
    } if repo.packages
    repo.patterns.each_value { |p|
      p.to_db( self, catalog )
    } if repo.patterns
    repo.products.each_value { |p|
      p.to_db( self, catalog )
    } if repo.products
    ActiveRecord::Base.connection.execute( "commit" )
  end

end # class Database

class Dependency

  def to_db type, db, resolvable
    return if @name[0,7] == "rpmlib("
    dep = Db::Dependency.new
    dep.name = @name
    dep.refers = Db::Reskind.get @refers.to_s
    dep.deptype = Db::Deptype.get type.to_s
    dep.resolvable = resolvable
    dep.pre = 1 if @pre
    if @op
	dep.relation = Db::Relation.get @op.to_s
	dep.version = @version
	dep.release = @release if @release
    end
    dep.save
  end

end

class Resolvable

  def dep_to_db dep, db, res
    return unless send(dep)
#puts "* saving #{dep} #{send(dep).class}"
    send(dep).each do |d|
#puts " saving #{d} #{d.class}"
      d.to_db dep, db, res
    end
  end


  def to_db db, catalog = nil
#puts "Dumping #{self.to_s}"
    res = Db::Resolvable.new
    res.kind = Db::Reskind.get @kind.to_s
    res.name = @name
    res.epoch = @epoch if @epoch
    res.version = @version
    res.release = @release if @release

    #raise "#{self.to_s} has no arch!" if res.architecture.nil?

    begin
      res.architecture = Db::Architecture.get @architecture.to_s
    rescue RuntimeError
      res.architecture = Db::Architecture.find(1)
    end
    res.catalog = catalog
    res.save

    dep_to_db :provides, db, res
    dep_to_db :requires, db, res
  end

end
