#
# dot.rb
#
# Output dependency graph as .dot format (for graphviz)
#

require "zypptools/graph/graph"

class Graph

  def to_dot channel, with_edgelabels = true
    channel.puts "digraph G {"
    channel.puts "  size=\"8.0, 11.0\";"
    channel.puts "  ratio=fill;"

# x -> y [style = {bold, solid, dotted, dashed}, label = ]

	@nodes.each_value do |node|
		channel.puts "# Node '#{node.to_s}'"
		case node.resolvable.kind
			when :package :	nodeshape = "ellipse"
			when :patch   : nodeshape = "diamond"
			when :pattern : nodeshape = "box"
			when :product : nodeshape = "house"
			else		nodeshape = "triangle"
		end
		channel.puts "node [ shape = #{nodeshape} ] \"#{node.name}\";"
	end
	@nodes.each_value do |node|
		edgelabel = ""
		node.outgoing.each do |edge|
			case edge.reason.kind
				when :requires : edgecolor = "black"
        when :provides
          next
				when :recommends, :supplements
				  next
          next if $ignore_weaks
          edgecolor = "blue"
				when :suggests, :enhances : edgestyle = "green"
				next
				else edgecolor = "red"
			end
			edgelabel = " [ color = \"#{edgecolor}\""
			edgelabel += ", label=\"#{edge.name}\"" if with_edgelabels
			edgelabel += "]"
			channel.puts "\"#{edge.from.name}\" -> \"#{edge.to.name}\"#{edgelabel};"
		end
	end

    channel.puts "}"
  end
end
