#
# helix.rb
#
# 'helix' (red carpet) xml file support
#
# adds save_helix( file ) functions everywhere
# (use file instead of returning a string to save memory)
#

require 'zypptools/lib/package'
require 'zypptools/lib/pattern'
require 'zypptools/lib/selection'
require 'zypptools/lib/product'
require 'zypptools/lib/dependency'

class Helix
	def Helix.save_helix( c, name, value, indent = 2 )
		return if value.nil?
		i1 = " " * indent
		if value.class == Array then
			c.puts i1 + "<#{name}>"
			value.each { |v|
				v.save_helix c, indent + 2
			}
			c.puts i1 + "</#{name}>"
		else
			c.puts i1 + "<#{name}>#{value}</#{name}>"
		end
	end

	def Helix.save_repo(repo, sel)
		puts "<channel><subchannel>"
		if (sel.empty? || sel.include?("packages")) then
			repo.packages.each_value { |p|
#				puts p
				p.save_helix STDOUT
			}
		end
		if (sel.empty? || sel.include?("patterns")) then
			repo.patterns.each_value { |p|
#				puts p
				p.save_helix STDOUT
			}
		end
		if (sel.empty? || sel.include?("products")) then
			repo.products.each_value { |p|
#				puts p
				p.save_helix STDOUT
			}
		end
		if (sel.empty? || sel.include?("selections")) then
			repo.selections.each_value { |p|
				p.save_helix STDOUT
			}
		end
		puts "</subchannel></channel>"
	end
end

class Dependency
	def save_helix( c, indent = 4 )
		return if @name[0,7] == "rpmlib("
		out = "<dep name='#{@name}'"
		out = out + " refers='#{@refers.id2name}'" if @refers
		if @op
			case @op
				when "=", "==": xmlop = @op
				when "<": xmlop = "&lt;"
				when ">": xmlop = "&gt;"
				when "<=": xmlop = "&lt;="
				when ">=": xmlop = "&gt;="
				else
					STDERR.puts "Unkown dep operation '#{@op}'"
			end
			out = out + " op='#{xmlop}' version='#{@version}'" if xmlop
			out = out + " release='#{@release}'" if @release
		end
		out += " pre='1'" if @pre
		out += " />"
		c.puts( (" " * indent) + out )
	end
end

class Package
	def save_helix( c, indent = 0 )
		c.puts "<package>"

		Helix.save_helix c, "name", name
		Helix.save_helix c, "summary", summary
		Helix.save_helix c, "description", description

		Helix.save_helix c, "section", @group.gsub( %r{/}, "_" ) if @group
		Helix.save_helix c, "location", @location

		c.puts "  <history>"
		c.puts "    <update>"

		Helix.save_helix c, "hid", "12345", 6
		Helix.save_helix c, "epoch", @epoch, 6
		Helix.save_helix c, "version", @version, 6
		Helix.save_helix c, "release", @release, 6
		Helix.save_helix c, "arch", @architecture, 6

		c.puts "    </update>"
		c.puts "  </history>"

		if prerequires || requires
			reqs = requires || []
			reqs += prerequires if prerequires
			Helix.save_helix c, "requires", (prerequires + requires)
		end
		Helix.save_helix c, "provides", provides
		Helix.save_helix c, "conflicts", conflicts
		Helix.save_helix c, "obsoletes", obsoletes
		Helix.save_helix c, "suggests", suggests
		Helix.save_helix c, "enhances", enhances
		Helix.save_helix c, "supplements", supplements
		Helix.save_helix c, "recommends", recommends

		c.puts "</package>"
	end
end

class Pattern
	def save_helix( c, indent = 0 )
		c.puts "<pattern>"

		Helix.save_helix c, "name", name
		Helix.save_helix c, "summary", summary
		Helix.save_helix c, "description", description

		c.puts "  <history>"
		c.puts "    <update>"

		Helix.save_helix c, "hid", "12345", 6
		Helix.save_helix c, "epoch", @epoch, 6
		Helix.save_helix c, "version", @version, 6
		Helix.save_helix c, "release", @release, 6
		Helix.save_helix c, "arch", @architecture, 6

		c.puts "    </update>"
		c.puts "  </history>"

		Helix.save_helix c, "requires", requires
		Helix.save_helix c, "provides", provides
		Helix.save_helix c, "conflicts", conflicts
		Helix.save_helix c, "obsoletes", obsoletes
		Helix.save_helix c, "suggests", suggests
		Helix.save_helix c, "enhances", enhances
		Helix.save_helix c, "supplements", supplements
		Helix.save_helix c, "recommends", recommends

		c.puts "</pattern>"
	end
end

class Selection
	def save_helix( c, indent = 0 )
		c.puts "<selection>"

		Helix.save_helix c, "name", name
		Helix.save_helix c, "summary", summary
		Helix.save_helix c, "description", description

		c.puts "  <history>"
		c.puts "    <update>"

		Helix.save_helix c, "hid", "12345", 6
		Helix.save_helix c, "epoch", @epoch, 6
		Helix.save_helix c, "version", @version, 6
		Helix.save_helix c, "release", @release, 6
		Helix.save_helix c, "arch", @architecture, 6

		c.puts "    </update>"
		c.puts "  </history>"

		Helix.save_helix c, "requires", requires
		Helix.save_helix c, "provides", provides
		Helix.save_helix c, "conflicts", conflicts
		Helix.save_helix c, "obsoletes", obsoletes
		Helix.save_helix c, "suggests", suggests
		Helix.save_helix c, "enhances", enhances
		Helix.save_helix c, "supplements", supplements
		Helix.save_helix c, "recommends", recommends

		c.puts "</selection>"
	end
end


class Product
	def save_helix( c, indent = 0 )
		c.puts "<product>"

		Helix.save_helix c, "name", name
		Helix.save_helix c, "summary", summary

		c.puts "  <history>"
		c.puts "    <update>"

		Helix.save_helix c, "hid", "12345", 6
		Helix.save_helix c, "epoch", @epoch, 6
		Helix.save_helix c, "version", @version, 6
		Helix.save_helix c, "release", @release, 6
		Helix.save_helix c, "arch", @architecture, 6

		c.puts "    </update>"
		c.puts "  </history>"

		Helix.save_helix c, "requires", requires
		Helix.save_helix c, "provides", provides
		Helix.save_helix c, "conflicts", conflicts
		Helix.save_helix c, "obsoletes", obsoletes
		Helix.save_helix c, "suggests", suggests
		Helix.save_helix c, "enhances", enhances
		Helix.save_helix c, "supplements", supplements
		Helix.save_helix c, "recommends", recommends

		c.puts "</product>"
	end
end



