require 'zlib'
require 'rexml/document'
include REXML 

#class for writing helix_patch-files

class Helix_out

	def initialize ( file_out, primary )
		@file = File.new(file_out, "a+")
		gz = Zlib::GzipReader.open(primary) 
		@primary = REXML::Document.new(gz.read)
	end

	# builds up the patch section
	def patch (p_name, p_arch, p_version, p_release, pa_name, pa_op, pa_version, pa_release, pa_epoch)
		i = 0
		@file << "<channel><subchannel>\n"
		@file << "\n<patch>\n"
		@file << "\t<name>#{p_name}</name>\n"
		@file << "\t<arch>#{p_arch}</arch>\n"
		@file << "\t<version>#{p_version}</version>\n"
		@file << "\t<release>#{p_release}</release>\n"
		@file << "\t<requires>\n"

	 	until pa_name[i] == nil
			@file << "\t\t<dep kind=\"atom\" name=\"#{pa_name[i]}\" "
			@file << "epoch=\"#{pa_epoch[i]}\" version=\"#{pa_version[i]}\" release=\"#{pa_release[i]}\" op=\"#{pa_op[i]}\"/>\n"
			i += 1
		end	

		@file << "\t</requires>\n" 
		@file << "</patch>\n"
	end

	# inserts all atoms
	def atom (a_name, a_arch, a_version, a_release, a_location, af_kind, af_name, ar_kind, ar_name, ar_op, ar_version, ar_release, ar_epoch)
		i = 0
		until a_name[i] == nil

			@file << "<atom>\n"
			@file << "\t<name>#{a_name[i]}</name>\n"
			@file << "\t<arch>#{a_arch[i]}</arch>\n"
			@file << "\t<version>#{a_version[i]}</version>\n"
			@file << "\t<release>#{a_release[i]}</release>\n"
			@file << "\t<freshens>\n"
			@file << "\t\t<dep kind=\"#{af_kind[i]}\" name=\"#{af_name[i]}\"/>\n"	
			@file << "\t</freshens>\n"
			@file << "\t<requires>\n"
			@file << "\t\t<dep kind=\"#{ar_kind[i]}\" name=\"#{ar_name[i]}\" epoch=\"#{ar_epoch[i]}\" version=\"#{ar_version[i]}\" "
			@file << "release=\"#{ar_release[i]}\" op=\"#{ar_op[i]}\"/>\n"
			@file << "\t</requires>\n"
			@file << "</atom>\n"
			i += 1	
		end
	end

	# inserts all packages
	def package (a_name, pac_sum, pac_desc, a_arch, a_version, a_release, pac_filesize, pac_instsize, pac_filename, ar_epoch, ar_op, pac_sec)
		i = 0
		until a_name[i] == nil
			@file << "<package>\n"
			@file << "\t<name>#{a_name[i]}</name>\n"
			@file << "\t<summary>#{pac_sum}</summary>\n"
			@file << "\t<description>#{pac_desc}</description>\n"
			@file << "\t<arch>#{a_arch[i]}</arch>\n"
			@file << "\t<version>#{a_version[i]}</version>\n"
			@file << "\t<release>#{a_release[i]}</release>\n"
			@file << "\t<filesize>#{pac_filesize[i]}</filesize>\n"
			@file << "\t<installedsize>#{pac_instsize[i]}</installedsize>\n"
			@file << "\t<history>\n"
			@file << "\t\t<update>\n"
			@file << "\t\t\t<arch>#{a_arch[i]}</arch>\n"
			@file << "\t\t\t<epoch>#{ar_epoch[i]}</epoch>\n"
			@file << "\t\t\t<version>#{a_version[i]}</version>\n"
			@file << "\t\t\t<release>#{a_release[i]}</release>\n"
			@file << "\t\t\t<filename>#{pac_filename[i]}</filename>\n"
			@file << "\t\t\t<filesize>#{pac_filesize[i]}</filesize>\n"
			@file << "\t\t\t<installedsize>#{pac_instsize[i]}</installedsize>\n"
			@file << "\t\t\t<importance>#{pac_sec}</importance>\n"
			#FIXME Is it necessary to include the description for every single package?
			#@file << "\t\t\t<description>#{pac_desc}</description>\n"
			@file << "\t\t\t<description>no desc</description>\n"
			@file << "\t\t</update>\n"
			@file << "\t</history>\n"
			@file << "\t<deps>\n"
			#invokes package_dep which will look for all dependencies, requieres ...
			package_dep(a_name[i], a_arch[i], a_version[i], a_release[i])
			@file << "\t</deps>\n"
			@file << "</package>\n"
			i += 1
		end
		@file << "</subchannel></channel>\n"
	end
	
	# searches for the deps of a certain package in primary.xml
	def package_dep (a_name, a_arch, a_version, a_release)
		@primary.elements.each("metadata") do |c|
			c.elements.each("package") do |cc|
				if cc.elements["name"].text == a_name && cc.elements["arch"].text == a_arch then
				    if cc.elements["version"].attributes["ver"] == a_version && cc.elements["version"].attributes["rel"] == a_release
					if cc.elements["format/rpm:provides"] then
						@file << "\t\t<provides>\n"
						cc.elements.each("format/rpm:provides") do |d|
							d.elements.each("rpm:entry") do |dd|
								@file << "\t\t\t<dep name=\"#{dd.attributes["name"]}\""
								if dd.attributes["ver"] then
								     @file << " op=\"#{dd.attributes["flags"]}\" version=\"#{dd.attributes["ver"]}\""
								     if dd.attributes["rel"] then
										@file << " release=\"#{dd.attributes["rel"]}\""
								     end
								end
								if dd.attributes["pre"] then
									@file << " pre=\"#{dd.attributes["pre"]}\""
								end
								@file << "/>\n"
							end
						end
						@file << "\t\t</provides>\n"		
					end				
					if cc.elements["format/rpm:requires"] then
						@file << "\t\t<requires>\n"
						cc.elements.each("format/rpm:requires") do |d|
							d.elements.each("rpm:entry") do |dd|
								@file << "\t\t\t<dep name=\"#{dd.attributes["name"]}\""
								if dd.attributes["ver"] then
							     	     @file << " op=\"#{dd.attributes["flags"]}\" version=\"#{dd.attributes["ver"]}\""
								     if dd.attributes["rel"] then
										@file << " release=\"#{dd.attributes["rel"]}\""
								     end
								end
								if dd.attributes["pre"] then
									@file << " pre=\"#{dd.attributes["pre"]}\""
								end
								@file << "/>\n"
							end
						end
						@file << "\t\t</requires>\n"
					end
					if cc.elements["format/rpm:conflicts"] then
						@file << "\t\t<conflicts>\n"
						cc.elements.each("format/rpm:conflicts") do |d|
							d.elements.each("rpm:entry") do |dd|
								@file << "\t\t\t<dep name=\"#{dd.attributes["name"]}\""
								if dd.attributes["ver"] then
							     	     @file << " op=\"#{dd.attributes["flags"]}\" version=\"#{dd.attributes["ver"]}\""
								     if dd.attributes["rel"] then
									@file << " release=\"#{dd.attributes["rel"]}\""
								     end
								end
								if dd.attributes["pre"] then
									@file << " pre=\"#{dd.attributes["pre"]}\""
								end
								@file << "/>\n"
							end
						end
						@file << "\t\t</conflicts>\n"
					end
				    end
				end
			end
		end
	end
end
