# Implements YUM metadata
# http://en.opensuse.org/Standards/Rpm_Metadata
# dmacvicar@suse.de

# Unfortunately this creates invalid xml: The closing metadata tag is
# placed before the packages.

require 'rubygems'
require 'builder'
require 'pathname'
require 'zlib'

class RpmMdOutput

  #
  #
  #
  def initialize( logger, path, compress = false )
    @logger = logger
    @path = path

    if ( File.exists?(@path+'/repodata') )
      if not ( File.directory?(@path+'/repodata') )
        raise "#{@path+'/repodata'} is not a directory"
      end
    else
      begin
        Dir.mkdir(@path+'/repodata')
      rescue Exception => e
        puts @path
        raise e
      end
      if not compress
        @primary_file = File.open(@path +'/repodata/primary.xml', 'w')
      else
        @primary_file = File.open(@path+'/repodata/primary.xml.gz', 'w')

      end
      raise "Can't open repodata dir" if @primary_file.nil?
    end

    # primary builder
    @primary_builder = Builder::XmlMarkup.new(:target => $stdout, :indent => 2)
    x = Builder::XmlMarkup.new(:target => $stdout, :indent => 2)
    x.instruct! :xml, :version=>"1.0", :encoding => "UTF-8"
    x.metadata :xmlns=>"http://linux.duke.edu/metadata/common", 'xmlns:rpm'.to_sym=>"http://linux.duke.edu/metadata/rpm", 'xmlns:suse'.to_sym=>"http://novell.com/package/metadata/suse/common"

  end

=begin
  <?xml version="1.0" encoding="UTF-8"?>
<metadata xmlns="http://linux.duke.edu/metadata/common" xmlns:rpm="http://linux.duke.edu/metadata/rpm" xmlns:suse="http://novell.com/package/metadata/suse/common" packages="18989">
=end

  def consume_package( p )
    puts "Repomd: consuming #{p}"
    x = @primary_builder
    x.package(:type => 'rpm') {
      x.name p.name
      x.arch p.arch
      x.version :epoch=> p.epoch, :ver=>p.version, :rel=>p.release
      x.checksum p.sha1sum, :type => "sha"
      x.summary p.summary
      x.description p.description
      x.packager p.packager
      x.url p.url
      x.time :file=>"1170984108", :build=>"1170983970"
      x.size :package=> File.size(p.filename), :installed=>p.size, :archive=>p.archivesize
      x.location :href=>p.filename
      x.format {
        x.rpm :license, p.license
        x.rpm :vendor, p.vendor
        x.rpm :group, p.group
        x.rpm :buildhost, p.host
        x.rpm :sourcerpm, p.sourcerpm
        # FIXME this range is wrong
        x.rpm 'header-range'.to_sym, :start=>"#{96 + p.sigsize}", :end=>"4955"
        [ :requires, :provides, :conflicts ].each do | dept |
          x.rpm(dept) {
            p.send( dept ).each do | cap |
              x.rpm :entry, :name=>cap.name, :flags=>cap.flag.to_s.upcase, :epoch=>cap.epoch, :ver=>cap.version, :rel=>cap.release#, :pre=>cap.pre
            end
          }
        end
      }
    }
  end

  def start
  end

  def consume( resolvable )
    puts resolvable.kind
    case resolvable.kind
      when :package
        puts "Processng package"
        consume_package( resolvable )
#@primary_file.flush
    end
  end

  def finish
    @primary_file.close
  end

end


