
# Output YaST Metadata
# aschnell@suse.de

require 'fileutils'
require 'zypptools/lib/utils'


module ZYppTools


 class YastOutput


  def initialize(path)

    @path = path
    @resolvables = Array.new()

    FileUtils.makedirs(@path + '/suse/setup/descr')
    FileUtils.makedirs(@path + '/media.1')

  end


  def start()
    @resolvables.clear()
  end

  def consume(resolvable)
    @resolvables.push(resolvable)
  end

  def finish()
    write(@resolvables)
  end


  def write_packages(resolvables)

    pkgs = resolvables
    pkgs.delete_if {|res| res.kind != :package}

    out = File.new(@path + "/suse/setup/descr/packages", 'w')

    out.puts "=Ver: 2.0"

    pkgs.each do |pkg|

      out.puts "##----------------------------------------"

      out.puts "=Pkg: #{pkg.name} #{pkg.version} #{pkg.release} #{pkg.arch}"

      if pkg.sha1sum
        out.puts "=Cks: SHA1 #{pkg.sha1sum}"
      end

      x = pkg.send(:requires)
      if x
        out.puts "+Req:"
        x.each do |cap|
          # FIXME: it this ok?
          if cap.op
            out.puts "#{cap.name} #{cap.op} #{cap.version}"
          else
            out.puts "#{cap.name}"
          end
        end
        out.puts "-Req:"
      end

      x = pkg.send(:provides)
      if x
        out.puts "+Prv:"
        x.each do |cap|
          # FIXME: it this ok?
          if cap.op
            out.puts "#{cap.name} #{cap.op} #{cap.version}"
          else
            out.puts "#{cap.name}"
          end
        end
        out.puts "-Prv:"
      end

      x = pkg.send(:supplements)
      if x
        out.puts "+Sup:"
        x.each do |cap|
          # FIXME: it this ok?
          if cap.op
            out.puts "#{cap.name} #{cap.op} #{cap.version}"
          else
            out.puts "#{cap.name}"
          end
        end
        out.puts "-Sup:"
      end

      if pkg.shared
        out.puts "=Shr: #{pkg.shared}"
      end

      if pkg.rpmsize and pkg.installsize
        out.puts "=Siz: #{pkg.rpmsize} #{pkg.installsize}"
      end

    end

    out.close()

  end


  def write_packages_en(resolvables)

    pkgs = resolvables
    pkgs.delete_if {|res| res.kind != :package}

    out = File.new(@path + "/suse/setup/descr/packages.en", 'w')

    out.puts "=Ver: 2.0"

    pkgs.each do |pkg|

      out.puts "##----------------------------------------"

      out.puts "=Pkg: #{pkg.name} #{pkg.version} 1 #{pkg.arch}"
      out.puts "=Sum: #{pkg.summary}"

      out.puts "+Des:"
      out.puts "-Des:"

    end

    out.close()

  end


  def write_media()
    out = File.new(@path + "/media.1/media", 'w')
    out.close()
  end


  def write_content()
    out = File.new(@path + "/content", 'w')
    # FIXME
    out.puts "PRODUCT SUSE LINUX"
    out.puts "VERSION 10.3"
    out.puts "DISTPRODUCT openSUSE-10.3"
    out.puts "DISTVERSION 10.3"
    out.puts "VENDOR SUSE LINUX Products GmbH, Nuernberg, Germany"
    out.puts "RELNOTESURL http://www.suse.com/relnotes/i386/SUSE-Linux/10.3/release-notes.rpm"
    out.puts "ARCH.i686 i686 i586 i486 i386 noarch"
    out.puts "ARCH.i586 i586 i486 i386 noarch"
    out.puts "ARCH.i486 i486 i386 noarch"
    out.puts "ARCH.i386 i386 noarch"
    out.puts "DEFAULTBASE i586"
    out.puts "REQUIRES openSUSE-release-10.3"
    out.puts "LINGUAS de en"
    out.puts "SHORTLABEL SL 10.3"
    out.puts "LABEL openSUSE 10.3"
    out.puts "LABEL.en openSUSE 10.3"
    out.puts "DESCRDIR suse/setup/descr"
    out.puts "DATADIR suse"
    out.puts "FLAGS update"
    out.puts "LANGUAGE en_US"
    out.puts "TIMEZONE America/Los_Angeles"
    out.puts "META SHA1 " + file_sha1sum(@path + "/suse/setup/descr/packages") + "  packages"
    out.puts "META SHA1 " + file_sha1sum(@path + "/suse/setup/descr/packages.en") + "  packages.en"
    out.close()
  end


  def write(resolvables)
    write_packages(resolvables)
    write_packages_en(resolvables)
    write_media()
    write_content()
  end


 end


end

