
# Output YUM Metadata
# Uses REXML, so might use lots of memory.
# aschnell@suse.de

require 'fileutils'
require 'zypptools/lib/utils'
require 'rexml/document'
include REXML


module ZYppTools

 class YumOutput


  def initialize(path)

    @path = path
    @resolvables = Array.new()

    FileUtils.makedirs(@path + '/repodata')

  end


  def start()
    @resolvables.clear()
  end

  def consume(resolvable)
    @resolvables.push(resolvable)
  end

  def finish()
    write(@resolvables)
  end


  def write_repomd()

    doc = Document.new
    doc << XMLDecl.new("1.0", "UTF-8")

    r = doc.add_element 'repomd', {"xmlns"=>"http://linux.duke.edu/metadata/repo"}

    d = r.add_element 'data', {"type"=>"primary"}
    d.add_element 'location', {"href"=>"repodata/primary.xml"}
    (d.add_element 'checksum', {"type"=>"sha"}).text = file_sha1sum(@path + "/repodata/primary.xml")
    (d.add_element 'open-checksum', {"type"=>"sha"}).text = file_sha1sum(@path + "/repodata/primary.xml")
    (d.add_element 'timestamp').text = file_mtime(@path + "/repodata/primary.xml")

    file = File.new(@path + "/repodata/repomd.xml", "w")
    doc.write(file, 0)
    file.close

  end


  def write_primary(resolvables)

    pkgs = resolvables
    pkgs.delete_if {|res| res.kind != :package}

    doc = Document.new
    doc << XMLDecl.new("1.0", "UTF-8")

    m = doc.add_element 'metadata', {"xmlns"=>"http://linux.duke.edu/metadata/common",
                                     "xmlns:rpm"=>"http://linux.duke.edu/metadata/rpm",
                                     "xmlns:suse"=>"http://novell.com/package/metadata/suse/common",
                                     "packages"=>pkgs.size}

    pkgs.each do |pkg|

      p = m.add_element 'package', {"type"=>"rpm"}
      (p.add_element 'name').text = pkg.name
      (p.add_element 'arch').text = pkg.arch
      (p.add_element 'version', {"epoch"=>pkg.epoch, "ver"=>pkg.version, "rel"=>pkg.release})
      (p.add_element 'checksum', {"type"=>"sha"}).text = pkg.sha1sum
      (p.add_element 'summary').text = pkg.summary
      (p.add_element 'description').text = pkg.description
      (p.add_element 'packager').text = pkg.packager
      (p.add_element 'url').text = pkg.url
      (p.add_element 'size', {"package"=>pkg.rpmsize, "installed"=>pkg.installsize})

      f = p.add_element 'format'
      (f.add_element 'rpm:license').text = pkg.license
      (f.add_element 'rpm:vendor').text = pkg.vendor
      (f.add_element 'rpm:group').text = pkg.group
      (f.add_element 'rpm:buildhost').text = pkg.buildhost
      (f.add_element 'rpm:sourcerpm').text = pkg.sourcerpm

      [ :requires, :provides, :conflicts, :supplements ].each do |dept|
        x = pkg.send(dept)
        if x
          d = f.add_element 'rpm:' + dept.to_s
          x.each do |cap|
            d.add_element 'rpm:entry', {"name"=>cap.name, "flags"=>cap.flag.to_s.upcase,
                                        "epoch"=>cap.epoch, "ver"=>cap.version,
                                        "rel"=>cap.release, #"pre"=>cap.pre
                                        }
          end
        end
      end
    end

    file = File.new(@path + "/repodata/primary.xml", "w")
    doc.write(file, 0)
    file.close

  end


  def write(resolvables)
    write_primary(resolvables)
    write_repomd()
  end


 end


end

