# zypp2 schema output
# dmacvicar@suse.de

require 'sqlite3'
require 'zypptools/lib/package'
require 'zypptools/lib/dependency'

class ZYppDatabaseBase
  def initialize( dbname, schema )
    @dbname = dbname
    exist = File.exists?(@dbname)
    @db = SQLite3::Database.new(@dbname)
    if not exist
      sql = File.new(schema).read
      @db.execute_batch( sql )
    end

  end

  def save_repo( repo )
    save_packages( repo )
  end

  def save_packages(repo)
    @db.execute("begin;")
    packages = repo.packages
    packages.each do | key, pkg |
      r_id = save_resolvable( pkg )
      #puts "#{r_id} #{pkg.name} #{pkg.edition}"
      i = 1
      values = Array.new
      [:resolvable_id,:license,:authors,:location,:package_group].each do | field |
        if field == :resolvable_id
          value = r_id
        else
          value = pkg.send(field)
        end
        values.push value
        #@insert_package_stmt.bind_params( i, value )
        i = i + 1
      end
      @insert_package_stmt.execute values
    end
    @db.execute("commit;")
  end

  def save_resolvable( resolv )
    i = 1
    values = Array.new
    [ :name, :version, :release, :epoch, :arch, :kind, :summary, :description ].each do | field |
      if field == :arch
        value = arch_to_db(resolv.send(field))
      elsif field == :kind
        value = kind_to_db(resolv.send(field))
      else
        value = resolv.send(field)
      end
      values.push value
      i = i + 1
    end
    @insert_resolvable_stmt.execute values
    id = @db.last_insert_row_id
    save_dependencies( resolv, id)
    return id
  end

  def save_dependencies( resolvable, resolvable_id )
    [ :requires, :provides, :conflicts, :obsoletes, :prerequires, :freshens, :recommends, :suggests, :supplements, :enhances ].each do |deptype|

      capset = resolvable.send(deptype)
      if not capset.nil?
        capset.each do | cap |
          @insert_dep_stmt.execute(resolvable_id, deptype_to_db(deptype), kind_to_db(cap.refers))
          capid = @db.last_insert_row_id
          save_versioned_capability(cap, capid)
        end
      end
    end
  end

  def deptype_to_db(deptype)
    n = [ :requires, :provides, :conflicts, :obsoletes, :prerequires, :freshens, :recommends, :suggests, :supplements, :enhances ].rindex(deptype)
    return n if not n.nil?
    return 42
  end
  # insert a name if it does not exists and returns id of new name or
  # the existing one
  def insert_name( name )
    v = @db.get_first_value("select id from names where name=?", name)
    if v.nil?
       @insert_name_stmt.execute name
       return @db.last_insert_row_id
    else
      return v
    end
  end

  def kind_to_db(kind)
     n = [ :package, :script, :message, :patch, :pattern, :product, :selection, :language, :atom, :src, :system ].rindex(kind.to_sym)
    return n if not n.nil?
    return 42
  end

  def arch_to_db(arch)
    n = [ :noarch, :i386, :i486, :i586, :i686, :x86_64, :ia32e, :athlon, :ppc, :ppc64, :s390, :s390x, :ia64, :sparc, :sparc64 ].rindex(arch.to_sym)
    return n if not n.nil?
    return -1
  end

  def rel_to_db(rel)
    n = [ :dummy, "==", "<", ">", "<=", ">=", "!=" ].rindex(rel)
    return n if not n.nil?
    return -1
  end
end

class ZYppDatabaseV2 < ZYppDatabaseBase
  def initialize(dbname)
    super( dbname, "/space/sources/zypp/trunk/libzypp/zypp2/cache/schema/schema_alt2.sql")
     @insert_resolvable_stmt = @db.prepare( "insert into resolvables ( name, version, release, epoch, arch, kind, summary, description) values ( ?, ?, ?, ?, ? ,? , ?, ? )" )
     @insert_package_stmt = @db.prepare( "insert into package_details (resolvable_id, license, authors, location, package_group) values (  ?, ?, ?, ?, ? )")
     #@insert_name_stmt = @db.prepare("insert into names (name) values (?)")
     #@insert_ver_capability_stmt = @db.prepare("insert into names (name) values (?)")
     @insert_dep_stmt = @db.prepare("insert into dependencies ( resolvable_id, dependency_type, refers_kind ) values ( ?, ?, ? )")
     @insert_ver_dep_stmt = @db.prepare("insert into versioned_dependencies ( dependency_id, name, version, release, epoch, relation ) values ( ?, ?, ?, ?, ?, ? )")
  end

  def save_versioned_capability( cap, dependency_id )
    #name_id = insert_name(cap.name)
    @insert_ver_dep_stmt.execute(dependency_id, cap.name, cap.version, cap.release, cap.epoch, rel_to_db(cap.op))

  end
end

class ZYppDatabase < ZYppDatabaseBase
  def initialize(dbname)
    super( dbname, "/space/sources/zypp/trunk/libzypp/zypp2/cache/schema/schema.sql")

    @insert_resolvable_stmt = @db.prepare( "insert into resolvables ( name, version, release, epoch, arch, kind, summary, description) values ( ?, ?, ?, ?, ? ,? , ?, ? )" )
    @insert_package_stmt = @db.prepare( "insert into package_details (resolvable_id, license, authors, location, package_group) values (  ?, ?, ?, ?, ? )")
    @insert_name_stmt = @db.prepare("insert into names (name) values (?)")
    @insert_name_stmt = @db.prepare("insert into names (name) values (?)")
    @insert_ver_capability_stmt = @db.prepare("insert into names (name) values (?)")
    @insert_dep_stmt = @db.prepare("insert into dependencies ( resolvable_id, dependency_type, refers_kind ) values ( ?, ?, ? )")
    @insert_ver_dep_stmt = @db.prepare("insert into versioned_dependencies ( dependency_id, name_id, version, release, epoch, relation ) values ( ?, ?, ?, ?, ?, ? )")
  end

  def save_versioned_capability( cap, dependency_id )
    name_id = insert_name(cap.name)
    @insert_ver_dep_stmt.execute(dependency_id, name_id, cap.version, cap.release, cap.epoch, rel_to_db(cap.op))

  end


end
